==> free-sa-1.4.5.tar.gz: free-sa-1.4.5/src/logformats/l_postfix_syslog.c <==
#include "global.h"


/* UNOPTIMIZED */
struct MailRecord *PFSLReadRecordI(FILE *iLog) {
    char daemonmod[16], tBuf[RECORDSIZE], tDT[SYSLOGDTSIZE];
    char *trs, *pstat;
    unsigned int tMsgid;
    long int tbytes;
    struct MailRecord *t = NULL;
    
    
    /* read next string from log */
    while (fgets(tBuf, sizeof(tBuf), iLog) != NULL) {
    	/* find beginning of string after message id */
	if ((trs = strchr(&tBuf[SYSLOGDTSIZE], ':')) == NULL) continue;
	if ((trs = strchr(trs + 1L, ':')) == NULL) continue; /* no messsage id */
	else trs += 2L;

	/* read datetime, daemon and module name, message id */
	if (sscanf(tBuf, "%" SYSLOGDTLEN "[^/] %*s %15[^[]%*s %8x%*[^:\n]", tDT, daemonmod, &tMsgid) != 3) continue;

	/* postfix daemon name */
	if (*(const int *)&daemonmod[0] != *(const int *)((const char *)"post") || 
	    *(const int *)&daemonmod[4] != *(const int *)((const char *)"fix/")) continue;

	/* find/create mail record by message id */
	t = FindNewMR(tMsgid);

	/* pickup module */
	if (*(const int *)&daemonmod[9] == *(const int *)((const char *)"icku")) {
	    t->status = 1;
	    strcpy(t->ip, "127.0.0.1");
	} else

	/* smtpd module */
	if (*(const int *)&daemonmod[9] == *(const int *)((const char *)"mtpd")) {
	    if (sscanf(trs, "client=%*[^[]%*c%" IPLEN "[^]]", t->ip) == 1) t->status = 1;
	    /* not check t->status == 1 because no nrcpt inforamtion available, 
	       postfix allow multiple rejects (often one by one) with same message id */
	    else if (sscanf(trs, "reject:%*[^[]%*c%" IPLEN "[^]]%*3c%hd%*[^;]%*[^<]%*c%" USERNAMELEN "[^>]%*[^<]%*c%" URLLEN "[^>]", t->ip, &t->code, t->name, t->url) == 4) {
		    t->bytes = 0;
		    SyslogDTZ(&t->time, tDT, 2007);
		    t->count = 1;
		    t->stat = 1;
		    t->status = 3;
		}
	} else

	/* qmgr module */
	if (*(const int *)&daemonmod[9] == *(const int *)((const char *)"mgr\000")) {
	    if (t->status == 1) {
		sscanf(trs, "from=<%" USERNAMELEN "[^>]", t->name);
		if (sscanf(trs, "%*[^>]>, size=%ld, nrcpt=%hd", &tbytes, (short int*)&t->count) == 2) {
		    t->bytes = (double)tbytes;
		    t->status = 2;
		}
	    }
	} else

	/* lmtp or smtp or pipe or local modules */
	if (*(const int *)&daemonmod[9] == *(const int *)((const char *)"mtp\000") || 
	    *(const int *)&daemonmod[9] == *(const int *)((const char *)"ipe\000") || 
	    *(const int *)&daemonmod[9] == *(const int *)((const char *)"ocal")) {
	    if (t->status == 2 && sscanf(trs, "to=<%" URLLEN "[^>]", t->url) == 1 && (pstat = strstr(trs, "status=")) != NULL) {
		switch (pstat[7]) {
		    case 's': t->code = 250; break; /* sent	*/
		    case 'd': t->code = 451; break; /* deferred */
		    case 'b': t->code = 550; break; /* bounced	*/
		     default: t->code = 600; SAwarning("Unknown stat: %c, please contact developers", t->stat); break;
		}

		SyslogDTZ(&t->time, tDT, 2007);
		t->stat = 0;
		t->status = 3;
	    }
	}

	if (t->status == 3) return t;
	else if (t->status == -1) DelMR(t);
    }

    return NULL;
}

==> free-sa-1.5.0.tar.gz: free-sa-1.5.0/src/logformats/l_postfix_syslog.c <==
#include "global.h"


/* UNOPTIMIZED */
struct MailRecord *PFSLReadRecordI(FILE *iLog) {
    char daemonmod[16], tBuf[RECORDSIZE], tDT[SYSLOGDTSIZE];
    char *trs, *pstat;
    unsigned int tMsgid;
    long int tbytes;
    struct MailRecord *t = NULL;
    
    
    /* read next string from log */
    while (fgets(tBuf, sizeof(tBuf), iLog) != NULL) {
    	/* find beginning of string after message id */
	if ((trs = strchr(&tBuf[SYSLOGDTSIZE], ':')) == NULL) continue;
	if ((trs = strchr(trs + 1L, ':')) == NULL) continue; /* no messsage id */
	else trs += 2L;

	/* read datetime, daemon and module name, message id */
	if (sscanf(tBuf, "%" SYSLOGDTLEN "[^/] %*s %15[^[]%*s %8x%*[^:\n]", tDT, daemonmod, &tMsgid) != 3) continue;

	/* postfix daemon name */
	if (*(const int *)&daemonmod[0] != *(const int *)((const char *)"post") || 
	    *(const int *)&daemonmod[4] != *(const int *)((const char *)"fix/")) continue;

	/* find/create mail record by message id */
	t = FindNewMR(tMsgid);

	/* pickup module */
	if (*(const int *)&daemonmod[9] == *(const int *)((const char *)"icku")) {
	    t->status = 1;
	    strcpy(t->ip, "127.0.0.1");
	} else

	/* smtpd module */
	if (*(const int *)&daemonmod[9] == *(const int *)((const char *)"mtpd")) {
	    if (sscanf(trs, "client=%*[^[]%*c%" IPLEN "[^]]", t->ip) == 1) t->status = 1;
	    /* not check t->status == 1 because no nrcpt inforamtion available, 
	       postfix allow multiple rejects (often one by one) with same message id */
	    else if (sscanf(trs, "reject:%*[^[]%*c%" IPLEN "[^]]%*3c%hd%*[^;]%*[^<]%*c%" USERNAMELEN "[^>]%*[^<]%*c%" URLLEN "[^>]", t->ip, &t->code, t->name, t->url) == 4) {
		    t->bytes = 0;
		    SyslogDTZ(&t->time, tDT, SYSLOGYEAR);
		    t->count = 1;
		    t->stat = 1;
		    t->status = 3;
		}
	} else

	/* qmgr module */
	if (*(const int *)&daemonmod[9] == *(const int *)((const char *)"mgr\000")) {
	    if (t->status == 1) {
		sscanf(trs, "from=<%" USERNAMELEN "[^>]", t->name);
		if (sscanf(trs, "%*[^>]>, size=%ld, nrcpt=%hd", &tbytes, (short int*)&t->count) == 2) {
		    t->bytes = (double)tbytes;
		    t->status = 2;
		}
	    }
	} else

	/* lmtp or smtp or pipe or local modules */
	if (*(const int *)&daemonmod[9] == *(const int *)((const char *)"mtp\000") || 
	    *(const int *)&daemonmod[9] == *(const int *)((const char *)"ipe\000") || 
	    *(const int *)&daemonmod[9] == *(const int *)((const char *)"ocal")) {
	    if (t->status == 2 && sscanf(trs, "to=<%" URLLEN "[^>]", t->url) == 1 && (pstat = strstr(trs, "status=")) != NULL) {
		switch (pstat[7]) {
		    case 's': t->code = 250; break; /* sent	*/
		    case 'd': t->code = 451; break; /* deferred */
		    case 'b': t->code = 550; break; /* bounced	*/
		     default: t->code = 600; SAwarning("Unknown stat: %c, please contact developers", t->stat); break;
		}

		SyslogDTZ(&t->time, tDT, SYSLOGYEAR);
		t->stat = 0;
		t->status = 3;
	    }
	}

	if (t->status == 3) return t;
	else if (t->status == -1) DelMR(t);
    }

    return NULL;
}
